
// audio stuff for butt
//
// Copyright 2007 by Daniel Noethen.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//


#include <stdio.h>
#include <stdlib.h>

#include <alsa/asoundlib.h>

#include "butt.h"
#include "alsa.h"
#include "parseconfig.h"
#include "vu-meter.h"
#include "flgui.h"

char *snd_pcm_buf;
extern unsigned char *alsa_buf;

int snd_send_status, snd_bytes_read;
snd_pcm_t *pcm_h = NULL;
snd_pcm_uframes_t buffer_size;
snd_pcm_uframes_t snd_chunk_size;
unsigned int chunk_bytes;

int snd_init()
{
    
	int rc;	
	int bytes_per_frame;
	int channel = 2;
	
	char info[100];
	
	snd_pcm_hw_params_t *pcm_params = NULL;	
	
   snd_bytes_read = 0;
   snd_send_status = 0;
   bytes_per_frame = 0;  
   snd_chunk_size = 0;
   snd_pcm_buf = NULL;
   
   

	/* Open PCM device for recording (capture). */
	rc = snd_pcm_open(&pcm_h, cfg.audio.pcm_dev,
                    SND_PCM_STREAM_CAPTURE, SND_PCM_NONBLOCK);
					// SND_PCM_STREAM_CAPTURE, 0);	

                      
	if (rc < 0) 
	{
		
		snprintf(info, sizeof(info), 
						"unable to open pcm device: %s\n%s", 
						cfg.audio.pcm_dev, 
						snd_strerror(rc));
			
		fl_g->print_info(info, 1);        
		return 1;
		
	} 
  
  /* Allocate a hardware parameters object. */
  	 snd_pcm_hw_params_alloca(&pcm_params);
  
	if (rc < 0) 
	{
				
		snprintf(info, sizeof(info), 
						"unable to allocate hw params memory:\n%s",						
						snd_strerror(rc));
			
		fl_g->print_info(info, 1);		
		return 1;
		
	} 

	/* Fill it in with default values. */
	rc = snd_pcm_hw_params_any(pcm_h, pcm_params);

	if (rc < 0) 
	{
				
		snprintf(info, sizeof(info), 
						"unable to set default hw params:\n%s",						
						snd_strerror(rc));
			
		fl_g->print_info(info, 1);
		return 1;
		
	}	
	
	/* Set the desired hardware parameters. */

	/* Interleaved mode */
	rc = snd_pcm_hw_params_set_access(pcm_h, pcm_params,
                      SND_PCM_ACCESS_RW_INTERLEAVED);
	if (rc < 0) 
	{
				
		snprintf(info, sizeof(info), 
						"unable to set pcm access mode:\n%s",						
						snd_strerror(rc));
			
		fl_g->print_info(info, 1);
		return 1;
	}
	
	
	/* Signed 16-bit little-endian format */
	rc = snd_pcm_hw_params_set_format(pcm_h, pcm_params, cfg.audio.format);
	if (rc < 0) 
	{
		
		snprintf(info, sizeof(info), 
						"unable to set pcm format: %d\n%s",
						cfg.audio.format,							
						snd_strerror(rc));
			
		fl_g->print_info(info, 1);
		return 1;
	}	

	/* set channels  (always capture stereo) */
	rc = snd_pcm_hw_params_set_channels(pcm_h, pcm_params, channel);
	
	if (rc < 0) 
	{
		        	
		snprintf(info, sizeof(info), 
						"unable to set num of pcm channels: %d\n%s",
						channel,							
						snd_strerror(rc));
			
		fl_g->print_info(info, 1);
		return 1;    
	}
	/* 44100 bits/second sampling rate (CD quality) */
	//val = 44100;
	rc = snd_pcm_hw_params_set_rate_near(pcm_h, pcm_params, 
                                  &cfg.audio.samplerate, 0);
	
	if (rc < 0) 
	{		
		snprintf(info, sizeof(info), 
						"unable to set pcm sampling rate: %d\n%s",
						cfg.audio.samplerate,							
						snd_strerror(rc));
			
		fl_g->print_info(info, 1);
		return 1;    
	}	

	
	buffer_size = 8192;
	snd_chunk_size = buffer_size / 4;
	
	rc = snd_pcm_hw_params_set_buffer_size_near(pcm_h, pcm_params, &buffer_size);	
	if (rc < 0) 
	{
		       
		snprintf(info, sizeof(info), 
						"unable to set buffer size: %lu\n%s",
						buffer_size,							
						snd_strerror(rc));
			
		fl_g->print_info(info, 1);
		return 1;   
	}
		
	rc = snd_pcm_hw_params_set_period_size_near(pcm_h, pcm_params, &snd_chunk_size, 0);
	if (rc < 0) 
	{
		
		snprintf(info, sizeof(info), 
						"unable to set period size: %lu\n%s",
						snd_chunk_size,							
						snd_strerror(rc));
			
		fl_g->print_info(info, 1);		
		return 1;   
	}
	
	
	/* Write the parameters to the driver */	
	rc = snd_pcm_hw_params(pcm_h, pcm_params);
	if (rc < 0) 
	{
		
		snprintf(info, sizeof(info), 
						"unable to write hw parameters to driver:\n%s",									
						snd_strerror(rc));
			
		fl_g->print_info(info, 1);
		return 1;   
	}
	
	rc = snd_pcm_hw_params_get_period_size(pcm_params, &snd_chunk_size, 0);
	if (rc < 0) 
	{
		
		snprintf(info, sizeof(info), 
						"unable to read period size:\n%s",									
						snd_strerror(rc));
			
		fl_g->print_info(info, 1);
		return 1;   
	}
	
	rc = snd_pcm_hw_params_get_buffer_size(pcm_params, &buffer_size);
	if (rc < 0) 
	{		
		snprintf(info, sizeof(info), 
						"unable to read buffer size:\n%s",									
						snd_strerror(rc));
			
		fl_g->print_info(info, 1);
		return 1;   
	}
	
	bytes_per_frame = cfg.audio.resolution * channel / 8;	
	
	chunk_bytes = snd_chunk_size * bytes_per_frame;
	

	snd_pcm_buf = (char*)malloc(chunk_bytes);
	encode_buf = (char*)malloc(chunk_bytes);	
	

	if(snd_pcm_buf == NULL)
	{
		
		snprintf(info, sizeof(info), 
						"not enough memory %s:%i",
						__FILE__, __LINE__);
			
			
		fl_g->print_info(info, 1);
		return 1;   
	}
	

	   		
	rc = snd_pcm_prepare(pcm_h);
	if (rc < 0) 
	{
				
         snprintf(info, sizeof(info), 
						"unable to prepare audio interface:\n%s",									
						snd_strerror(rc));
			
		fl_g->print_info(info, 1);
		return 1;   
	}

	return 0;
}

void snd_loop()
{
	

	for(;;)
	{
        snd_capture(snd_pcm_buf, snd_chunk_size);
		Fl::check();
    }		
	

}



int snd_capture(char* buf, int buf_len)
{
		
	ssize_t read;
	size_t result = 0;
	size_t count = buf_len;

	while (count > 0) 
	{
		
		read = snd_pcm_readi(pcm_h, buf, count);		
	
		if (read == -EAGAIN || (read >= 0 && (size_t)read < count)) //not enough data  
		{				
			snd_pcm_wait(pcm_h, 1000);			//wait max. 1 sec for new data
		}
		else if (read == -EPIPE) //buffer overrun
		{
			if(snd_send_status)
				fl_g->xrun->value(1);
				
			snd_pcm_prepare(pcm_h);
		} 
	
		else if (read < 0) 
		{
			fprintf(stderr, "read error: %s", snd_strerror(read));
			exit(EXIT_FAILURE);
		}
		if (read > 0) 
		{
			
			result += read;
			count -= read;
			buf += read;
		}
		
		for(int i = 0; i < read; i +=4)
		{
		
			vu_meter((snd_pcm_buf[i+1] <<8) + snd_pcm_buf[i], 	   //left
					 (snd_pcm_buf[i+3] <<8) + snd_pcm_buf[i+2]);   //right 
					 
		}		

	}
		
	return result;
}

snd_dev_t **snd_get_devices(int *dev_num)
{
    
    int i;    
    int card = -1;
    int pcm_device = -1;
    char *p;
    char *card_name;    
    int snd_devices = 1;
    
    snd_dev_t **dev_list;
    
    snd_pcm_info_t *pcm_info;
    snd_ctl_t *ctl;

    
    //reserve mem space for 20 card devices that should be enough
    //otherwise we will realloc more mem space later
    dev_list = (snd_dev_t**)malloc(20*sizeof(snd_dev_t*));
    
    for(i = 0; i < 20; i++)
        dev_list[i] = (snd_dev_t*)malloc(sizeof(snd_dev_t));   
        
 
    sprintf(dev_list[0]->name, "Default PCM device [default]");
    sprintf(dev_list[0]->dev, "default");
    
    snd_card_next(&card);
    while(card > -1)
    {
        
        sprintf(dev_list[snd_devices]->dev, "hw:%i", card);
        snd_ctl_open(&ctl, dev_list[snd_devices]->dev, 0);
        snd_card_get_name(card, &card_name);
        snprintf(dev_list[snd_devices]->name, 255, card_name);        
             
      	snd_pcm_info_alloca(&pcm_info);

        
        snd_ctl_pcm_next_device(ctl, &pcm_device);
        
        while(pcm_device > -1)
        {
            
            snd_pcm_info_set_device(pcm_info, pcm_device);
            snd_pcm_info_set_subdevice(pcm_info, 0);
            snd_pcm_info_set_stream(pcm_info, SND_PCM_STREAM_CAPTURE);
                        
            if (snd_ctl_pcm_info(ctl, pcm_info) < 0)
            {                
                snd_ctl_pcm_next_device(ctl, &pcm_device);
                continue;
            }

               
                     
            
            sprintf(dev_list[snd_devices]->dev, "hw:%d,%d", card, pcm_device);
            sprintf(dev_list[snd_devices]->name, "%s: %s [%s]",
                                               card_name,
                                               snd_pcm_info_get_name(pcm_info),
                                               dev_list[snd_devices]->dev
                                               );
            
              while((p = strchr(dev_list[snd_devices]->name, '/')) != NULL)
            {                
                //TODO: Comment this
                *p = '-';
                
            }
                                             
            snd_ctl_pcm_next_device(ctl, &pcm_device);
            snd_devices++;
            
        }
        
        snd_ctl_close(ctl);
        snd_card_next(&card);        
        
    }    
    *dev_num = snd_devices;
    
    return dev_list;
}


void snd_close()
{
	free(snd_pcm_buf);
    free(encode_buf);
	snd_pcm_drain(pcm_h);
	snd_pcm_close(pcm_h);		
}

