// callback stuff for butt
//
// Copyright 2007 by Daniel Noethen.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//

#include <FL/Fl_File_Chooser.H>

#include <errno.h>
#include <unistd.h>
#include <signal.h>
#include <sys/wait.h>

#include "flgui.h"
#include "network.h"
#include "vu-meter.h"
#include "encode.h"
#include "parseconfig.h"


void flgui::button_connect_cb(void)
{
	
	if(snd_send_status)
		return;
		
    int encode_bytes_read = 0;
    int encode_bytes_read_rec = 0;
	char info_text_buf[128];
	char lcd_text_buf[33];
	snd_bytes_read = 0;	
	int rc, connected = 0, dummy = 0, sec = 0, min = 0, hour = 0;
	time_t time_new, time_start;


    if(cfg.main.num_of_srv < 1)
    {
        fl_g->print_info("There must be at least one server defined.\n"
                         "Click on the red button right to the LCD to\n"
                         "add a server", 1);
        return;
        
    }
    
	read_cfg_widgets();
    
	sprintf(info_text_buf, "Connecting to %s:%u (%u) ...", cfg.srv[cfg.selected_srv]->addr, cfg.srv[cfg.selected_srv]->port+1, cfg.srv[cfg.selected_srv]->port); 
	//sprintf(info_text_buf, "connecting to shoutcast.server.org:8000 ..."); //for screenshots ;-)
	
	print_info(info_text_buf, 0);	

	time_start = time(NULL);
	network_bytes_sent = 0;
	
    //the user may not change the sound device while streaming
    choice_cfg_dev->deactivate();
    
	snd_send_status = 1;
	while(snd_send_status)
	{
		
		Fl::check();
		
		if(xrun->value())
			xrun->value(0);
		
		if(!connected)
		{
			while(rc = network_conn())			
			{
				if( (rc == 2) || (snd_send_status == 0) ) 
				{
					snd_send_status = 0;					//invalid password or the user has pressed the stop button 
					return;
				}				
				
				switch(dummy)
				{
				
				case 0:
					lcd->clear();
					lcd->print((const uchar*)"connecting", 10);
					dummy++;
					break;
				case 10:
				case 20:	
				case 30:
					lcd->print((const uchar*)".", 1);
					dummy++;
					break;
				case 40:
					dummy = 0;
					break;
				default:
					dummy++;					
				}
				
				Fl::check();
				
			}
			
			connected = 1;			
			print_info("connected", 0);
		}	

		snd_bytes_read = snd_capture(snd_pcm_buf, snd_chunk_size);
        	
		if(	snd_bytes_read > 0)
		{
		            
            if(record)
            {					
				encode_bytes_read_rec = encode_mp3(snd_pcm_buf, encode_buf, snd_bytes_read, gfp_record);			
				fwrite(encode_buf, 1, encode_bytes_read_rec, cfg.rec.fd);                    
            }
            
            encode_bytes_read = encode_mp3(snd_pcm_buf, encode_buf, snd_bytes_read, gfp_stream);
            rc = network_send(encode_buf, encode_bytes_read);
			if(encode_bytes_read > 0)
			{								
			
				if(rc < 0) //disconnected from host
				{				
					print_info("ERROR: server has closed the connection\nreconnecting ...", 1);
													
					close(network_s);
					connected = 0;
				}
				else
				{					
					network_bytes_sent += rc;
					
					time_new = time(NULL);
					if(time_new != time_start)
					{
					
						sec = time_new - time_start;
												
						min = sec / 60;
						hour = min / 60;	
						min %= 60;						
						sec %= 60;						
						
					}					
					
					if(display_info == DATA)			
						sprintf(lcd_text_buf, "status: on air\nsent: %ikb", network_bytes_sent / 1024);
					if(display_info == TIME)			
						sprintf(lcd_text_buf, "status: on air\ntime: %02i:%02i:%02i", hour, min, sec);						

					lcd->clear();
					lcd->print((const uchar*)lcd_text_buf, strlen(lcd_text_buf));				
                    
				}
				
			}		
		}			
	}
	
	//if(connected)		
		close(network_s);
	
	xrun->value(0);	
	lcd->clear();
	lcd->print((const uchar*)"status: idle", 12);

		

}

void flgui::button_disconnect_cb(void)
{

		if(snd_send_status)
		{
			print_info("user disconnected\n", 0);
			snd_send_status = 0;		
			lcd->clear();
			lcd->print((const uchar*)"status: idle", 12);
			xrun->value(0);	

            choice_cfg_dev->activate();


        }
		
}

void flgui::button_record_cb(void)
{
	
	
	int rc = 0, cancel = 0;
	char mode[3];
	
	if(record)
	{		
		fclose(cfg.rec.fd);
		record = 0;
		print_info("recording deactivated", 0);
		
	}
	else
	{		
		cfg.rec.filename = fl_file_chooser("select file for recording", 
			"mp3 files (*.mp3)\togg files (*.ogg)\tpcm data (*.pcm)", NULL);
		if(cfg.rec.filename != NULL) 							 	//check if a valid file was selected
		{			
			if(fopen(cfg.rec.filename, "rb") != NULL)				//check if the file already exists
			{
				rc = fl_choice("file already exists!", 
								"cancel", "overwrite", "append");
				
				switch(rc)
				{				
					case 0:															//cancel pressed
						cancel = 1;					
						break;	
					case 1:															//overwrite pressed
						sprintf(mode, "wb");									
						break;
					
					case 2: 														//append pressed
						sprintf(mode, "ab");
				}					
			}
			else																	//selected file doesnt exist yet
				sprintf(mode, "wb");			
			
			if(cancel == 0)
				if( (cfg.rec.fd = fopen(cfg.rec.filename, mode)) != NULL)
				{
					record = 1;
					print_info("recording activated\nrecording to:", 0);
					print_info(cfg.rec.filename, 0);
				}
				else
					fl_alert("could not open:\n%s", cfg.rec.filename);
			
			
		} //if(audio_settings.record.filename != NULL)			
	
		
	} 

}



void flgui::button_cfg_cb()
{
 
    if(window_cfg->shown())
    {
        window_cfg->hide();
        button_cfg->label("@>");
        
    }
    else
    {
  
        window_cfg->position(window_main->x() + window_main->w(), window_main->y());
        window_cfg->show();
        button_cfg->label("@<");
        
        fill_cfg_widgets();        
    }
    
    
    
}

//This part is mostly from the flcdsim project written by
//Michael Sweet

void flgui::button_info_cb()
{
	
  if (!info_visible)
  {
    // Show info output...
    window_main->resize(window_main->x(), window_main->y(), window_main->w(),
                    info_output->y() + 180);
    info_output->show();
    button_info->label("Info @2<");
	
	info_visible = 1;

  }
  else
  {
    // Hide info output...
    window_main->resize(window_main->x(), window_main->y(), window_main->w(),
                    info_output->y());
    info_output->hide();
    button_info->label("Info @2>");
	info_visible = 0;    

  }
}

void flgui::read_cfg_widgets()
{
   

  
    
   
    
}




void flgui::fill_cfg_widgets()
{
    
    unsigned int i;
    char dummy[50];    
	int dev_num;
    
    unsigned int bitrate[15] = { 32, 40, 48, 56, 64, 80, 96, 112, 128, 160, 192, 224, 256, 320, 0 };
    
    //fill the main section
    for(i = 0; i < cfg.main.num_of_srv; i++)
    {
        choice_cfg_act_srv->add(cfg.srv[i]->name);
        choice_cfg_edit_srv->add(cfg.srv[i]->name);
    }     
    
    for(i = 0; i < cfg.main.num_of_icy; i++)
    {
        choice_cfg_act_icy->add(cfg.icy[i]->name);
        choice_cfg_edit_icy->add(cfg.icy[i]->name);
    }   
        
    choice_cfg_act_srv->value(cfg.selected_srv);
    choice_cfg_act_icy->value(cfg.selected_icy);
    
    //fill the server sections
    if(cfg.main.num_of_srv > 0)    
    {
        choice_cfg_edit_srv->value(cfg.selected_srv);    
        
        input_cfg_addr->value(cfg.srv[cfg.selected_srv]->addr);
    
        snprintf(dummy, 6, "%u", cfg.srv[cfg.selected_srv]->port);
        input_cfg_port->value(dummy);    
        input_cfg_passwd->value(cfg.srv[cfg.selected_srv]->pwd);
    }
       
    //fill the audio section         
    cfg.audio.pcm_list = snd_get_devices(&dev_num);
    
    for(i = 0; (int)i < dev_num; i++)    
    {
        choice_cfg_dev->add(cfg.audio.pcm_list[i]->name);
        if(!strcmp(cfg.audio.pcm_dev, cfg.audio.pcm_list[i]->dev))
            choice_cfg_dev->value(i);        
    }
    
    
    
    if(!strcmp(cfg.audio.codec, "mp3"))
    {
            radio_cfg_codec_mp3->value(1);
            radio_cfg_codec_ogg->value(0);
    }
    else if(!strcmp(cfg.audio.codec, "ogg"))
    {
           radio_cfg_codec_ogg->value(1);
           radio_cfg_codec_mp3->value(0);
    }
    
    if(cfg.audio.channel == 1)
    {
            radio_cfg_channel_mono->value(1);
            radio_cfg_channel_stereo->value(0);
    }
    else if(cfg.audio.channel == 2)
    {
            radio_cfg_channel_stereo->value(1);
            radio_cfg_channel_mono->value(0);
    }
    
         
    for(i = 0; bitrate[i] != 0; i++)
        if(cfg.audio.bitrate == bitrate[i])
            choice_cfg_bitrate->value(i);     
      
    //fill the icy sections
    if(cfg.main.num_of_icy> 0)
    {
        choice_cfg_edit_icy->value(cfg.selected_icy);
    
        input_cfg_desc->value(cfg.icy[cfg.selected_icy]->desc);
        input_cfg_genre->value(cfg.icy[cfg.selected_icy]->genre);
        input_cfg_url->value(cfg.icy[cfg.selected_icy]->url);
        input_cfg_irc->value(cfg.icy[cfg.selected_icy]->irc);
        input_cfg_icq->value(cfg.icy[cfg.selected_icy]->icq);
        input_cfg_aim->value(cfg.icy[cfg.selected_icy]->aim);
    
        if(!strcmp(cfg.icy[cfg.selected_icy]->pub, "1"))
            check_cfg_pub->value(1);
        else 
            check_cfg_pub->value(0);
    }    
    
    
    
}

void flgui::choice_cfg_act_srv_cb()
{
 
    cfg.selected_srv = choice_cfg_act_srv->value();    
    
    cfg.main.srv = (char*)realloc(cfg.main.srv, strlen(cfg.srv[cfg.selected_srv]->name)+1);    
    sprintf(cfg.main.srv, cfg.srv[cfg.selected_srv]->name);
       
}
void flgui::choice_cfg_act_icy_cb()
{
 
    cfg.selected_icy = choice_cfg_act_icy->value();
    
    cfg.main.icy = (char*)realloc(cfg.main.icy, strlen(cfg.icy[cfg.selected_icy]->name)+1);    
    sprintf(cfg.main.icy, cfg.icy[cfg.selected_icy]->name);  
}


void flgui::button_cfg_add_srv_cb()
{
    
    window_add_srv->position(window_cfg->x(), window_cfg->y());
    
    //give the "name" input field the input focus
    input_add_srv_name->take_focus();
    
    window_add_srv->show();
    
    
}
void flgui::button_cfg_add_icy_cb()
{
    
    window_add_icy->position(window_cfg->x(), window_cfg->y());
    
    //give the "name" input field the input focus
    input_add_icy_name->take_focus();
    
    window_add_icy->show();
    
    
}


void flgui::button_cfg_sav_cb()
{
    
}

void flgui::button_cfg_srv_apply_cb()
{
 
    int srv_num = choice_cfg_edit_srv->value();
     
    //update current server address
    
    cfg.srv[srv_num]->addr = 
        (char*) realloc(cfg.srv[srv_num]->addr, 
                sizeof(char) * strlen(input_cfg_addr->value())+1);
        
    sprintf(cfg.srv[srv_num]->addr, input_cfg_addr->value());
    
    //update current server port
    cfg.srv[srv_num]->port = (unsigned int)atoi(input_cfg_port->value());
    
    
    //update current server password
    
    cfg.srv[srv_num]->pwd = 
        (char*) realloc(cfg.srv[srv_num]->pwd, 
                    strlen(input_cfg_passwd->value())+1);
    
    sprintf(cfg.srv[srv_num]->pwd, input_cfg_passwd->value());   
    
    
}

void flgui::button_cfg_icy_apply_cb()
{
    
    int icy_num = choice_cfg_edit_icy->value();

       
    cfg.icy[icy_num]->desc = 
        (char*)realloc(cfg.icy[icy_num]->desc, 
                  strlen(input_cfg_desc->value())+1 );        
    sprintf(cfg.icy[icy_num]->desc, input_cfg_desc->value());
    
    cfg.icy[icy_num]->genre = 
        (char*)realloc(cfg.icy[icy_num]->genre, 
                  strlen(input_cfg_genre->value())+1 );    
    sprintf(cfg.icy[icy_num]->genre, input_cfg_genre->value());
    
    cfg.icy[icy_num]->url = 
        (char*)realloc(cfg.icy[icy_num]->url, 
                  strlen(input_cfg_url->value())+1 );    
    sprintf(cfg.icy[icy_num]->url, input_cfg_url->value());
    
    cfg.icy[icy_num]->icq = 
        (char*)realloc(cfg.icy[icy_num]->icq, 
                  strlen(input_cfg_icq->value())+1 );    
    sprintf(cfg.icy[icy_num]->icq, input_cfg_icq->value());
    
    cfg.icy[icy_num]->irc = 
        (char*)realloc(cfg.icy[icy_num]->irc, 
                  strlen(input_cfg_irc->value())+1 );    
    sprintf(cfg.icy[icy_num]->irc, input_cfg_irc->value());
    
    cfg.icy[icy_num]->aim = 
        (char*)realloc(cfg.icy[icy_num]->aim, 
                  strlen(input_cfg_aim->value())+1 );    
    sprintf(cfg.icy[icy_num]->aim, input_cfg_aim->value());
    
    sprintf(cfg.icy[icy_num]->pub, "%d", check_cfg_pub->value());
        
    
    
}

void flgui::choice_cfg_edit_srv_cb()
{

    char dummy[10];
    int server = choice_cfg_edit_srv->value();
    
            
    input_cfg_addr->value(cfg.srv[server]->addr);
    
    snprintf(dummy, 6, "%u", cfg.srv[server]->port);
    input_cfg_port->value(dummy);    
    input_cfg_passwd->value(cfg.srv[server]->pwd);
    
    
    
}

void flgui::choice_cfg_cat_cb()
{

    unsigned int bitrate[15] = { 32, 40, 48, 56, 64, 80, 96, 112, 128, 160, 192, 224, 256, 320, 0 };
    unsigned int i;
    
    
    if(choice_cfg_cat->value() == 0)
    {
        
        if(!strcmp(cfg.audio.codec, "mp3"))
        {
            radio_cfg_codec_mp3->value(1);
            radio_cfg_codec_ogg->value(0);
        }
        else if(!strcmp(cfg.audio.codec, "ogg"))
        {
           radio_cfg_codec_ogg->value(1);
           radio_cfg_codec_mp3->value(0);
        }
    
        if(cfg.audio.channel == 1)
        {
            radio_cfg_channel_mono->value(1);
            radio_cfg_channel_stereo->value(0);
        }
        else if(cfg.audio.channel == 2)
        {
            radio_cfg_channel_stereo->value(1);
            radio_cfg_channel_mono->value(0);
        }
    
         
        for(i = 0; bitrate[i] != 0; i++)
            if(cfg.audio.bitrate == bitrate[i])
                choice_cfg_bitrate->value(i);
            
            
    }
    else if(choice_cfg_cat->value() == 1)
    {
        
        if(!strcmp(cfg.rec.codec, "mp3"))
        {
            radio_cfg_codec_mp3->value(1);
            radio_cfg_codec_ogg->value(0);
        }
        else if(!strcmp(cfg.rec.codec, "ogg"))
        {
            radio_cfg_codec_ogg->value(1);
            radio_cfg_codec_mp3->value(0);
        }
    
        if(cfg.rec.channel == 1)
        {
            radio_cfg_channel_mono->value(1);
            radio_cfg_channel_stereo->value(0);
        }
        else if(cfg.rec.channel == 2)
        {
            radio_cfg_channel_stereo->value(1);
            radio_cfg_channel_mono->value(0);
        }
    
         
        for(i = 0; bitrate[i] != 0; i++)
            if(cfg.rec.bitrate == bitrate[i])
                choice_cfg_bitrate->value(i);
    }
    
    
}

void flgui::choice_cfg_bitrate_cb()
{

    unsigned int sel_bitrate;
    unsigned int bitrate[14] = { 32, 40, 48, 56, 64, 80, 96, 112, 128, 160, 192, 224, 256, 320 };
    
        
    sel_bitrate = choice_cfg_bitrate->value();
    
    
    
    if(choice_cfg_cat->value() == 0)                //streaming
    {
        cfg.audio.bitrate = bitrate[sel_bitrate];
        
        free(cfg.icy[cfg.selected_icy]->br);
        cfg.icy[cfg.selected_icy]->br = (char*)malloc(4*sizeof(char));
        snprintf(cfg.icy[cfg.selected_icy]->br, 4, "%d", cfg.audio.bitrate);
        
    }
    else if (choice_cfg_cat->value() == 1)          //recording
        cfg.rec.bitrate = bitrate[sel_bitrate];
    
    //this callback func gets called if the user changes the bitrate value
    //so we need to reinit the encode handlers with the new bitrate
    encode_close();
    encode_init();    
    
}

void flgui::radio_cfg_channel_stereo_cb()
{
    if(choice_cfg_cat->value() == 0)                //streaming
        cfg.audio.channel = 2;
    else if (choice_cfg_cat->value() == 1)          //recording
        cfg.rec.channel = 2;
    
    encode_close();
    encode_init();    
    
}

void flgui::radio_cfg_channel_mono_cb()
{
    
    if(choice_cfg_cat->value() == 0)                //streaming
        cfg.audio.channel = 1;
    else if (choice_cfg_cat->value() == 1)          //recording
        cfg.rec.channel = 1;
    
    encode_close();
    encode_init();   
    
}
 
void flgui::button_add_srv_ok_cb()
{
    
    int i;   
    
    //error checking
    if(strlen(input_add_srv_name->value()) == 0)
    {
        fl_alert("No name specified");
        return;
    }
    if(strlen(input_add_srv_addr->value()) == 0)
    {
        fl_alert("No address specified");
        return;
    }
    if(strlen(input_add_srv_pwd->value()) == 0)
    {
        fl_alert("No password specified");
        return;
    }
    if(strlen(input_add_srv_port->value()) == 0)
    {
        fl_alert("No port specified");
        return;
    }
    else if(( atoi(input_add_srv_port->value()) > 65535) || (atoi(input_add_srv_port->value()) < 0) )
    {
        fl_alert("Invalid portnumber\nThe portnumber may range from 0 to 65535");
        return;
        
    }
    
    
    //I use the "i" here just to make the following lines more readable    
    i = cfg.main.num_of_srv;
    cfg.main.num_of_srv++;    
        
    cfg.srv = (server_t**)realloc(cfg.srv, cfg.main.num_of_srv * sizeof(server_t*));
    cfg.srv[i] = (server_t*)malloc(sizeof(server_t));
    
    cfg.srv[i]->name = (char*)malloc(strlen(input_add_srv_name->value())+1);
    sprintf(cfg.srv[i]->name, input_add_srv_name->value());
    
    cfg.srv[i]->addr = (char*)malloc(strlen(input_add_srv_addr->value())+1);
    sprintf(cfg.srv[i]->addr, input_add_srv_addr->value());
    
    cfg.srv[i]->pwd = (char*)malloc(strlen(input_add_srv_pwd->value())+1);
    sprintf(cfg.srv[i]->pwd, input_add_srv_pwd->value());    
    
    cfg.srv[i]->port = (unsigned int)atoi(input_add_srv_port->value());
        
    
    if(cfg.main.num_of_srv > 1)
    {
        cfg.main.srv_ent = (char*)realloc(cfg.main.srv_ent, 
                                         strlen(cfg.main.srv_ent) +
                                         strlen(cfg.srv[i]->name) +2);
        sprintf(cfg.main.srv_ent, "%s;%s", cfg.main.srv_ent, cfg.srv[i]->name);
    }
    else
    {
        cfg.main.srv_ent = (char*)malloc(strlen(cfg.srv[i]->name) +1);
        sprintf(cfg.main.srv_ent, "%s", cfg.srv[i]->name);
    }
    
    if(cfg.main.srv == NULL)
        cfg.main.srv = (char*)malloc(strlen(cfg.srv[cfg.selected_srv]->name)+1);
    else
        cfg.main.srv = (char*)realloc(cfg.main.srv, strlen(cfg.srv[cfg.selected_srv]->name)+1);
    
    sprintf(cfg.main.srv, cfg.srv[cfg.selected_srv]->name);
    
    //reset the input fields and hide the window
    input_add_srv_name->value("");
    input_add_srv_addr->value("");
    input_add_srv_port->value("");
    input_add_srv_pwd->value("");
    
    window_add_srv->hide();
    
    fill_cfg_widgets();
    
}

void flgui::button_add_srv_cancel_cb()
{
    
    input_add_srv_name->value("");
    input_add_srv_addr->value("");
    input_add_srv_port->value("");
    input_add_srv_pwd->value("");
    
    window_add_srv->hide();
    
}

void flgui::button_add_icy_ok_cb()
{
    
    int i;
    //error checking
    if(strlen(input_add_icy_name->value()) == 0)
    {
        fl_alert("No name specified");
        return;
    }
    
    //I use the "i" here just to make the following lines more readable    
    i = cfg.main.num_of_icy;    
    cfg.main.num_of_icy++;
        
    cfg.icy = (icy_t**)realloc(cfg.icy, cfg.main.num_of_icy * sizeof(icy_t*));
    cfg.icy[i] = (icy_t*)malloc(sizeof(icy_t));       
    
    cfg.icy[i]->name = (char*)malloc(strlen(input_add_icy_name->value())+1 );
    sprintf(cfg.icy[i]->name, input_add_icy_name->value());
    
    cfg.icy[i]->desc = (char*)malloc(strlen(input_add_icy_desc->value())+1 );
    sprintf(cfg.icy[i]->desc, input_add_icy_desc->value());
    
    cfg.icy[i]->url = (char*)malloc(strlen(input_add_icy_url->value())+1 );
    sprintf(cfg.icy[i]->url, input_add_icy_url->value());
    
    cfg.icy[i]->genre = (char*)malloc(strlen(input_add_icy_genre->value())+1 );
    sprintf(cfg.icy[i]->genre, input_add_icy_genre->value());
    
    cfg.icy[i]->irc = (char*)malloc(strlen(input_add_icy_irc->value())+1 );
    sprintf(cfg.icy[i]->irc, input_add_icy_irc->value());
    
    cfg.icy[i]->icq = (char*)malloc(strlen(input_add_icy_icq->value())+1 );
    sprintf(cfg.icy[i]->icq, input_add_icy_icq->value());
    
    cfg.icy[i]->aim = (char*)malloc(strlen(input_add_icy_aim->value())+1 );
    sprintf(cfg.icy[i]->aim, input_add_icy_aim->value());
    
    cfg.icy[i]->pub = (char*)malloc(2 * sizeof(char));
    sprintf(cfg.icy[i]->pub, "%d", check_add_icy_pub->value());
    
    cfg.icy[i]->br = (char*)malloc(4 * sizeof(char));
    sprintf(cfg.icy[i]->br, "%d", cfg.audio.bitrate);
    
     if(!strcmp(cfg.audio.codec, "mp3"))
     {
         cfg.icy[i]->type = (char*)malloc(strlen("audio/mpeg\0"));								
         sprintf(cfg.icy[i]->type, "audio/mpeg");	
     }
     
    
    if(cfg.main.num_of_icy > 1)
    {
        cfg.main.icy_ent = (char*)realloc(cfg.main.icy_ent, 
                                         strlen(cfg.main.icy_ent) +
                                         strlen(cfg.icy[i]->name) +2);
        sprintf(cfg.main.icy_ent, "%s;%s", cfg.main.icy_ent, cfg.icy[i]->name);
    }
    else
    {
        cfg.main.icy_ent = (char*)malloc(strlen(cfg.icy[i]->name) +1);
        sprintf(cfg.main.icy_ent, "%s", cfg.icy[i]->name);
    }
    
    if(cfg.main.icy == NULL)
        cfg.main.icy = (char*)malloc(strlen(cfg.icy[cfg.selected_icy]->name)+1);
    else
        cfg.main.icy = (char*)realloc(cfg.main.icy, strlen(cfg.icy[cfg.selected_icy]->name)+1);
    
    sprintf(cfg.main.icy, cfg.icy[cfg.selected_icy]->name);
        
    input_add_icy_name->value("");
    input_add_icy_desc->value("");
    input_add_icy_url->value("");
    input_add_icy_genre->value("");
    input_add_icy_irc->value("");
    input_add_icy_icq->value("");
    input_add_icy_aim->value("");
    check_add_icy_pub->value(0);
    
    window_add_icy->hide();
    
    fill_cfg_widgets();
    
}

void flgui::button_add_icy_cancel_cb()
{
    
    input_add_icy_name->value("");
    input_add_icy_desc->value("");
    input_add_icy_url->value("");
    input_add_icy_genre->value("");
    input_add_icy_irc->value("");
    input_add_icy_icq->value("");
    input_add_icy_aim->value("");
    check_add_icy_pub->value(0);
    
    window_add_icy->hide();
    
}




void flgui::choice_cfg_edit_icy_cb()
{
    
    
    int icy = choice_cfg_edit_icy->value();
    
    input_cfg_desc->value(cfg.icy[icy]->desc);
    input_cfg_genre->value(cfg.icy[icy]->genre);
    input_cfg_url->value(cfg.icy[icy]->url);
    input_cfg_irc->value(cfg.icy[icy]->irc);
    input_cfg_icq->value(cfg.icy[icy]->icq);
    input_cfg_aim->value(cfg.icy[icy]->aim);
    
    
    
    if(!strcmp(cfg.icy[icy]->pub, "1"))
        check_cfg_pub->value(1);
    else 
        check_cfg_pub->value(0);
                
}

void flgui::button_cfg_del_srv_cb()
{
    
    int i;
    int diff;
    
    if(cfg.main.num_of_srv == 0)
        return;
    
    diff = cfg.main.num_of_srv-1 - cfg.selected_srv;
    
    for(i = 0; i < diff; i++)
    {
        memcpy(cfg.srv[cfg.selected_srv+i], cfg.srv[cfg.selected_srv+i+1], sizeof(server_t));   
        
        
    }
    
    free(cfg.srv[cfg.main.num_of_srv-1]);    
    
    
    cfg.main.num_of_srv--;
    cfg.selected_srv = 0;  
    
    //rearrange the string that contains all server names
    memset(cfg.main.srv_ent, 0, strlen(cfg.main.srv_ent));    
    printf(cfg.main.srv_ent);
 /*   for(i = 0; i < (int)cfg.main.num_of_srv; i++)
    {
        strcat(cfg.main.srv_ent, cfg.srv[i]->name);
        
        //the last entry doesn't have a trailing seperator ";"
        if(i < (int)cfg.main.num_of_srv-1)
            strcat(cfg.main.srv_ent, ";");
        
    }*/

    
    //clear the server choicemenu in the main and server section
    //fill_cfg_widgets() will refill them    
    choice_cfg_act_srv->clear();
    choice_cfg_act_srv->redraw();           //Yes we need this :-(
    
    choice_cfg_edit_srv->clear();
    choice_cfg_act_srv->redraw();
    
    
    fill_cfg_widgets();
    
    
}

void flgui::button_cfg_del_icy_cb()
{
    
    int i;
    int diff;
    
    if(cfg.main.num_of_icy == 0)
        return;
    
    diff = cfg.main.num_of_icy-1 - cfg.selected_icy;
    
    for(i = 0; i < diff; i++)
    {
        memcpy(cfg.icy[cfg.selected_icy+i], cfg.icy[cfg.selected_icy+i+1], sizeof(icy_t));          
        
    }
    
    free(cfg.icy[cfg.main.num_of_icy-1]);    
    
    cfg.main.num_of_icy--;
    cfg.selected_icy = 0;
    
     //rearrange the string that contains all ICY names
    memset(cfg.main.icy_ent, 0, strlen(cfg.main.icy_ent));    
    for(i = 0; i < (int)cfg.main.num_of_icy; i++)
    {
        strcat(cfg.main.icy_ent, cfg.icy[i]->name);
        
        //the last entry doesn't have a trailing seperator ";"
        if(i < (int)cfg.main.num_of_icy-1)
            strcat(cfg.main.icy_ent, ";");
        
    }
    
    //clear the ICY choicemenu in the main and ICY section
    //fill_cfg_widgets() will refill them    
    choice_cfg_act_icy->clear();
    choice_cfg_act_icy->redraw();           //Yes we need this :-(
    
    choice_cfg_edit_icy->clear();
    choice_cfg_act_icy->redraw(); 
    
    
    fill_cfg_widgets();
    
    
    
}

void flgui::choice_cfg_dev_cb()
{
    int ent;
    ent = choice_cfg_dev->value();
    
    cfg.audio.pcm_dev = (char*) realloc(cfg.audio.pcm_dev, 
                                strlen(cfg.audio.pcm_list[ent]->dev)+1); 
    
  
    sprintf(cfg.audio.pcm_dev, cfg.audio.pcm_list[ent]->dev);
    snd_close();
    snd_init();
    //snd_loop();
    
    
}

