/*
 * config file parser
 *
 */

//taken from camE http://linuxbrit.co.uk/camE/

 
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "butt.h"
#include "encode.h"
#include "alsa.h"
#include "flgui.h"
#include "parseconfig.h"

struct CFG_ENTRIES {
    unsigned int  ent_count;
    char **ent_names;
    char **ent_values;
};

struct CFG_SECTIONS {
    unsigned int                 sec_count;
    char                **sec_names;
    struct CFG_ENTRIES  **sec_entries;
};

/* ------------------------------------------------------------------------ */

static struct CFG_SECTIONS *c;

/* ------------------------------------------------------------------------ */

#define ALLOC_SIZE 16

static struct CFG_SECTIONS*
cfg_init_sections()
{
    struct CFG_SECTIONS *c;
    c = (CFG_SECTIONS*) malloc(sizeof(struct CFG_SECTIONS));
    memset(c,0,sizeof(struct CFG_SECTIONS));
    c->sec_names      = (char**) malloc(ALLOC_SIZE*sizeof(char*));
    c->sec_names[0]   = NULL;
    c->sec_entries    = (CFG_ENTRIES**) malloc(ALLOC_SIZE*sizeof(struct CFG_ENTRIES*));
    c->sec_entries[0] = NULL;
    return c;
}

static struct CFG_ENTRIES*
cfg_init_entries()
{
    struct CFG_ENTRIES *e;
    e = (CFG_ENTRIES*) malloc(sizeof(struct CFG_ENTRIES));
    memset(e,0,sizeof(struct CFG_ENTRIES));
    e->ent_names     = (char**) malloc(ALLOC_SIZE*sizeof(char*));
    e->ent_names[0]  = NULL;
    e->ent_values    = (char**) malloc(ALLOC_SIZE*sizeof(char*));
    e->ent_values[0] = NULL;
    return e;
}

static struct CFG_ENTRIES*
cfg_find_section(struct CFG_SECTIONS *c, char *name)
{
    struct CFG_ENTRIES* e;
    unsigned int i;

    for (i = 0; i < c->sec_count; i++)
	if (0 == strcmp(c->sec_names[i],name))
	    return c->sec_entries[i];

    /* 404 not found => create a new one */
    if ((c->sec_count % ALLOC_SIZE) == (ALLOC_SIZE-2)) {
	c->sec_names   = (char**) realloc(c->sec_names,(c->sec_count+2+ALLOC_SIZE)*sizeof(char*));
	c->sec_entries = (CFG_ENTRIES**) realloc(c->sec_entries,(c->sec_count+2+ALLOC_SIZE)*sizeof(struct CFG_ENTRIES*));
    }
    e = cfg_init_entries();
    c->sec_names[c->sec_count]   = strdup(name);
    c->sec_entries[c->sec_count] = e;
    c->sec_count++;    
    c->sec_names[c->sec_count]   = NULL;
    c->sec_entries[c->sec_count] = NULL;
    return e;
}

static void
cfg_set_entry(struct CFG_ENTRIES *e, char *name, char *value)
{
    unsigned int i;	
	
	

    for (i = 0; i < e->ent_count; i++)
	if (0 == strcmp(e->ent_names[i],name))
	    break;
    if (i == e->ent_count) {
	/* 404 not found => create a new one */
	if ((e->ent_count % ALLOC_SIZE) == (ALLOC_SIZE-2)) {
	    e->ent_names  = (char**) realloc(e->ent_names,(e->ent_count+2+ALLOC_SIZE)*sizeof(char*));
	    e->ent_values = (char**) realloc(e->ent_values,(e->ent_count+2+ALLOC_SIZE)*sizeof(char*));
	}
	e->ent_count++;    
	e->ent_names[e->ent_count]  = NULL;
	e->ent_values[e->ent_count] = NULL;
    }
    e->ent_names[i]  = strdup(name);
    e->ent_values[i] = strdup(value);
}

/* ------------------------------------------------------------------------ */

int
cfg_parse_file(char *filename)
{
    struct CFG_ENTRIES *e = NULL;
    char line[256],tag[64],value[192];
    FILE *fp;
    int nr;
    
    if (NULL == c)
	c = cfg_init_sections();
    if (NULL == (fp = fopen(filename,"r")))
	return -1;

    nr = 0;
    while (NULL != fgets(line,255,fp)) {
	nr++;
    if (line[0] == '\n' || line[0] == '#' || line[0] == '%' || line[0] ==';')
	    continue;
	if (1 == sscanf(line,"[%99[^]]]",value)) {
	    /* section */
	    e = cfg_find_section(c,value);
	} else if (2 == sscanf(line," %63[^= ] = %191[^\n]",tag,value)) {
	    /* foo = bar */
	    if (NULL == e) {
		fprintf(stderr,"%s:%d: error: no section\n",filename,nr);
	    } else {			
		cfg_set_entry(e,tag,value);
	    }
	} /*else {
	   // Huh ? 
	    fprintf(stderr,"%s:%d: syntax error\n",filename,nr);
	}*/
    }
    fclose(fp);
    return 0;
}

/* ------------------------------------------------------------------------ */

char**
cfg_list_sections()
{
    return c->sec_names;
}

char**
cfg_list_entries(char *name)
{
    unsigned int i;
    
    for (i = 0; i < c->sec_count; i++)
	if (0 == strcmp(c->sec_names[i],name))
	    return c->sec_entries[i]->ent_names;
    return NULL;
}

char*
cfg_get_str(char *sec, char *ent)
{
    struct CFG_ENTRIES* e = NULL;
    char *v = NULL;
    unsigned int i;
    
    for (i = 0; i < c->sec_count; i++)
	if (0 == strcmp(c->sec_names[i],sec))
	    e = c->sec_entries[i];
    if (NULL == e)
	return NULL;
    for (i = 0; i < e->ent_count; i++)
	if (0 == strcmp(e->ent_names[i],ent))
	    v = e->ent_values[i];
    return v;
}

int
cfg_get_int(char *sec, char *ent)
{
    char *val;

    val = cfg_get_str(sec,ent);
    if (NULL == val)
	return -1;
    return atoi(val);
}

float
cfg_get_float(char *sec, char *ent)
{
    char *val;

    val = cfg_get_str(sec,ent);
    if (NULL == val)
	return -1;
    return atof(val);
}


int cfg_write_file()
{
    
        
    FILE *cfg_fd;
    unsigned int i;
    char info_dummy[200];
    
  
   cfg_fd = fopen(cfg_path, "wb+");   
   if(cfg_fd == NULL)
   {
       sprintf(info_dummy, "Could not write to file: %s", cfg_path);
       fl_g->print_info(cfg_path, 1);
       return 1;       
   }
   
   fprintf(cfg_fd, "This is a configuration file for butt (broadcast using this tool)\n\n");
   
   fprintf(cfg_fd, "[main]\n");
   
   if(cfg.main.num_of_srv > 0)
       fprintf(cfg_fd, "server = %s\n"
                       "srv_ent = %s\n",           
                        cfg.main.srv,
                        cfg.main.srv_ent
                        );
   else
       fprintf(cfg_fd, "server = \n"
                       "srv_ent = \n"
                       );   
   
   
   if(cfg.main.num_of_icy > 0)
       fprintf(cfg_fd, "icy = %s\n"
                       "icy_ent = %s\n",           
                        cfg.main.icy,
                        cfg.main.icy_ent
                        );
   else
       fprintf(cfg_fd, "icy = \n"
                       "icy_ent = \n"                            
                       );
   
   fprintf(cfg_fd, "num_of_srv = %d\n"
                   "num_of_icy = %d\n\n",
                   cfg.main.num_of_srv,
                   cfg.main.num_of_icy
                   );
      
   
                
   fprintf(cfg_fd, "[audio]\n"
                   "device = %s\n"
                   "samplerate = %d\n"
                   "bitrate = %d\n"
                   "channel = %d\n"
                   "codec = %s\n\n",
                   cfg.audio.pcm_dev,
                   cfg.audio.samplerate,
                   cfg.audio.bitrate, 
                   cfg.audio.channel,
                   cfg.audio.codec
                   );
    
    fprintf(cfg_fd, "[record]\n"
                    "bitrate = %d\n"
                    "channel = %d\n"
                    "codec = %s\n\n",
                    cfg.rec.bitrate,
                    cfg.rec.channel,
                    cfg.rec.codec
                    );
    
    for(i = 0; i < cfg.main.num_of_srv; i++)
    {
        
        fprintf(cfg_fd, "[%s]\n" 
                        "address = %s\n"
                        "port = %u\n"
                        "password = %s\n\n",            
                        cfg.srv[i]->name,
                        cfg.srv[i]->addr,
                        cfg.srv[i]->port,
                        cfg.srv[i]->pwd
                        );
        
    }
    
    for(i = 0; i < cfg.main.num_of_icy; i++)
    {
        
        fprintf(cfg_fd, "[%s]\n" 
                        "description = %s\n"
                        "genre = %s\n"
                        "url = %s\n"
                        "irc = %s\n"
                        "icq = %s\n"
                        "aim = %s\n"
                        "pub = %s\n\n",
                        cfg.icy[i]->name,
                        cfg.icy[i]->desc,
                        cfg.icy[i]->genre,
                        cfg.icy[i]->url,
                        cfg.icy[i]->irc,
                        cfg.icy[i]->icq,
                        cfg.icy[i]->aim,
                        cfg.icy[i]->pub
                        );
        
    }
    
    
    
    fclose(cfg_fd);
    
    fl_g->print_info("config written to ~/.buttrc", 0);
    
    return 0;
}


int cfg_set_values()
{
	
    unsigned int i;
    char *srv_ent;
    char *icy_ent;
    char *strtok_buf;
    char info[100];
    
    
	    
	if(cfg_parse_file(cfg_path) == -1)
	{
		snprintf(info, sizeof(info), "Missing config file %s", cfg_path);
		fl_g->print_info(info, 1);
		
		return 1;
	}
	
	
	
	//TODO: add error handling
	
	cfg.audio.pcm_dev =  cfg_get_str("audio", "device");	
	cfg.audio.samplerate = cfg_get_int("audio", "samplerate");

	cfg.audio.resolution = 16;	
	cfg.audio.bitrate = cfg_get_int("audio", "bitrate");
	
	cfg.audio.channel = cfg_get_int("audio", "channel");   	
	cfg.audio.format = SND_PCM_FORMAT_S16_LE;                 //Stereo signed 16bit little endian
	
    cfg.audio.codec =  	cfg_get_str("audio", "codec");	
	
    
	cfg.rec.bitrate = cfg_get_int("record", "bitrate");		
	cfg.rec.channel = cfg_get_int("record", "channel");	
    cfg.rec.codec = cfg_get_str("record", "codec");
	cfg.rec.filename = cfg_get_str("record", "file");	
    
    cfg.main.num_of_srv = cfg_get_int("main", "num_of_srv");
    if(cfg.main.num_of_srv > 0)
    {
        cfg.main.srv = cfg_get_str("main", "server");
        cfg.main.srv_ent = cfg_get_str("main", "srv_ent");          //server entries
    
        cfg.srv = (server_t**)malloc(sizeof(server_t*) * cfg.main.num_of_srv);
    
        for(i = 0; i < cfg.main.num_of_srv; i++)        
            cfg.srv[i] = (server_t*)malloc(sizeof(server_t));    
	
        srv_ent = (char*)malloc(MAX_SECTION_LENGTH+1 * sizeof(char));   
      
        //strdupa uses alloca instead of malloc
        //so the string doesnt need to be free'd
        strtok_buf = strdupa(cfg.main.srv_ent);
        
        srv_ent = strtok(strtok_buf, ";");
        for(i = 0; srv_ent != NULL; i++)    
        {
            cfg.srv[i]->name = (char*)malloc(MAX_SECTION_LENGTH+1 * sizeof(char));
       
            snprintf(cfg.srv[i]->name, MAX_SECTION_LENGTH, srv_ent);
            cfg.srv[i]->addr =  cfg_get_str(srv_ent, "address");
            cfg.srv[i]->port = cfg_get_int(srv_ent, "port");
            cfg.srv[i]->pwd = cfg_get_str(srv_ent, "password");
       
            if(!strcmp(cfg.srv[i]->name, cfg.main.srv))
                cfg.selected_srv = i;
               
            srv_ent = strtok(NULL, ";");     
        }
    }// if(cfg.main.num_of_srv > 0)
    
    cfg.main.num_of_icy = cfg_get_int("main", "num_of_icy");

    if(cfg.main.num_of_icy > 0)
    {
        cfg.main.icy = cfg_get_str("main", "icy");    
        cfg.main.icy_ent = cfg_get_str("main", "icy_ent");          //icy entries    
    
        cfg.icy = (icy_t**)malloc(sizeof(icy_t*) * cfg.main.num_of_icy);    
        
        for(i = 0; i < cfg.main.num_of_icy; i++)        
            cfg.icy[i] = (icy_t*)malloc(sizeof(icy_t));    
	
        icy_ent = (char*)malloc(MAX_SECTION_LENGTH+1 * sizeof(char));
   
        strtok_buf = strdupa(cfg.main.icy_ent);
   
        icy_ent = strtok(strtok_buf, ";");
      
        for(i = 0; icy_ent != NULL; i++)    
        {
            cfg.icy[i]->name = (char*)malloc(MAX_SECTION_LENGTH+1 * sizeof(char));       
            snprintf(cfg.icy[i]->name, MAX_SECTION_LENGTH, icy_ent);
       
            cfg.icy[i]->desc = cfg_get_str(icy_ent, "description");
            cfg.icy[i]->genre = cfg_get_str(icy_ent, "genre");
            cfg.icy[i]->url = cfg_get_str(icy_ent, "url");
            cfg.icy[i]->irc = cfg_get_str(icy_ent, "irc");
            cfg.icy[i]->icq = cfg_get_str(icy_ent, "icq");
            cfg.icy[i]->aim = cfg_get_str(icy_ent, "aim");
            cfg.icy[i]->pub = cfg_get_str(icy_ent, "pub");	
            cfg.icy[i]->br = cfg_get_str("audio", "bitrate");
       
            if(!strcmp(cfg.audio.codec, "mp3"))
            {
                cfg.icy[i]->type = (char*)malloc(strlen("audio/mpeg\0"));								
                sprintf(cfg.icy[i]->type, "audio/mpeg");	
            }
            
            if(!strcmp(cfg.icy[i]->name, icy_ent)) 
                cfg.selected_icy = i;
               
            icy_ent = strtok(NULL, ";");
        }
    }//if(cfg.main.num_of_icy > 0)

   

	return 0;
}

int cfg_create_default()
{
    FILE *cfg_fd;    
    cfg_fd = fopen(cfg_path, "wb+");
    if(cfg_fd == NULL)
        return 1;
   
   fprintf(cfg_fd, "This is a configuration file for butt (broadcast using this tool)\n\n");
   fprintf(cfg_fd, "[main]\n"
                   "server = \n"
                   "icy = \n"
                   "num_of_srv = 0\n"
                   "num_of_icy = 0\n"
                   "srv_ent = \n"
                   "icy_ent = \n\n"                 
                   );
                
   fprintf(cfg_fd, "[audio]\n"
                   "device = default\n"
                   "samplerate = 44100\n"
                   "bitrate = 128\n"
                   "channel = 2\n"
                   "codec = mp3\n\n"                   
                   );
    
    fprintf(cfg_fd, "[record]\n"
                    "bitrate = 192\n"
                    "channel = 2\n"
                    "codec = mp3\n\n"                    
                    );    
    
    fclose(cfg_fd);
    
    return 0;
}







